@@ -1,6 +1,6 @@
 from typing import Any, Union
 
-from apispec import BasePlugin
+from apispec import BasePlugin, APISpec
 from pydantic import BaseModel
 
 
@@ -25,12 +25,16 @@ class PydanticPlugin(BasePlugin):
     def schema_helper(
         self, name: str, definition: dict, **kwargs: Any
     ) -> Union[dict, None]:
-        model: Union[BaseModel, None] = kwargs.pop("model")
+        model: Union[BaseModel, None] = kwargs.pop("model", None)
         if model:
             schema = model.schema(ref_template="#/components/schemas/{model}")
 
-            # Pydantic doesn't know that we are aggregating all of our models separately,
-            # so we need to delete its attempts at being smart.
+            # If the spec has passed, we probably have nested models to contend with.
+            spec: Union[APISpec, None] = kwargs.pop("spec", None)
+            if spec and "definitions" in schema:
+                for (k, v) in schema["definitions"].items():
+                    spec.components.schema(k, v)
+
             if "definitions" in schema:
                 del schema["definitions"]
 
